/*============================================================================
	File: SQLskillsLogInfo.sql 

	Summary: This script creates a system-wide SP SQLskillsLogInfo that
	displays log usage information as well as the number of VLFs in the log
	file for all databases on an instance 

	Date: January 2010 

	SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
	Copyright (C) 2010 Paul S. Randal, SQLskills.com
	All rights reserved. 

	For more scripts and sample code, check out
		http://www.sqlskills.com/ 

	You may alter this code for your own *non-commercial* purposes. You may
	republish altered code as long as you give due credit. 

	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/ 

-- Create the function in msdb
--
--:setvar SQLCMDLOGINTIMEOUT 60 
--:setvar server "MyServer" 
--:connect $(server) -l $(SQLCMDLOGINTIMEOUT) 

USE msdb;
GO 

IF EXISTS (SELECT * FROM sys.objects WHERE NAME = 'sp_SQLskillsAllLogInfo')
	DROP PROCEDURE sp_SQLskillsAllLogInfo;
GO

-- This SP reports log info for all databases on an instance:
-- 
-- EXEC sp_SQLskillsAllLogInfo;
-- GO
--
-- Database Name  Recovery Model  Log Size (MB)  Log Used (%)  Log File Count  VLF Count  Log Rebuilds  Last Log Backup
-- -------------- --------------- -------------- ------------- --------------- ---------- ------------- -----------------------
-- master         SIMPLE          6.12           34.42         1               22         0             NEVER
-- tempdb         SIMPLE          0.49           75.60         1               2          0             NEVER
-- model          FULL            14.68          97.90         1               51         0             2007-08-09 16:14:20.450
-- msdb           SIMPLE          2.49           45.92         1               10         0             NEVER
-- adventureworks SIMPLE          1.99           33.55         1               4          0             2009-05-19 14:02:10.920
-- SalesDB        FULL            359.99         77.84         1               96         0             2009-05-24 13:26:59.907
-- DBMaint2008    FULL            0.48           45.67         1               2          0             NEVER
--
--
CREATE PROCEDURE sp_SQLskillsAllLogInfo
AS
BEGIN
	SET NOCOUNT ON; 

	-- Create the temp tables
	--
	IF EXISTS (SELECT * FROM msdb.sys.objects WHERE NAME = 'SQLskillsAllInfo')
		DROP TABLE msdb.dbo.SQLskillsAllInfo; 

	IF EXISTS (SELECT * FROM msdb.sys.objects WHERE NAME = 'SQLskillsLogSpace')
		DROP TABLE msdb.dbo.SQLskillsLogSpace; 

	IF EXISTS (SELECT * FROM msdb.sys.objects WHERE NAME = 'SQLskillsLogInfo')
		DROP TABLE msdb.dbo.SQLskillsLogInfo; 

	IF EXISTS (SELECT * FROM msdb.sys.objects WHERE NAME = 'SQLskillsDBCC')
		DROP TABLE msdb.dbo.SQLskillsDBCC; 

	IF EXISTS (SELECT * FROM msdb.sys.objects WHERE NAME = 'SQLskillsSYSDB')
		DROP TABLE msdb.dbo.SQLskillsSYSDB; 

	CREATE TABLE msdb.dbo.SQLskillsAllInfo (
		[DatabaseName]	VARCHAR (128),
		[LogSize]		FLOAT,
		[LogUsed]		FLOAT,
		[FileCount]		INT,
		[VLFCount]		INT,
		[Rebuilds]		INT,
		[RecModel]		VARCHAR (15),
		[LLBackup]		VARCHAR (30)); 

	CREATE TABLE msdb.dbo.SQLskillsLogSpace (
		[DatabaseName]	VARCHAR (128),
		[LogSize]		FLOAT,
		[LogUsed]		FLOAT,
		[Status]		INT);

	CREATE TABLE msdb.dbo.SQLskillsLogInfo (
		[FileId]		INT,
		[FileSize]		BIGINT,
		[StartOffset]	BIGINT,
		[FSeqNo]		INT,
		[Status]		INT,
		[Parity]		TINYINT,
		[LSN]			VARCHAR (100));		

	CREATE TABLE msdb.dbo.SQLskillsDBCC (
		[ParentObject]	VARCHAR (100),
		[Object]		VARCHAR (100),
		[Field]			VARCHAR (100),
		[VALUE]			VARCHAR (100)); 

	-- Get the DBCC SQLPERF (LOGSPACE) info and transfer to
	-- the aggregate table
	--
	INSERT INTO msdb.dbo.SQLskillsLogSpace
	EXEC ('DBCC SQLPERF (LOGSPACE)');

	INSERT msdb.dbo.SQLskillsAllInfo (
		[DatabaseName], [LogSize], [LogUsed], [VLFCount])
	SELECT * FROM msdb.dbo.SQLskillsLogSpace;

	-- Declare variables
	--
	DECLARE @VLFs		INT;
	DECLARE @FCount		INT;
	DECLARE @LRCount	INT;
	DECLARE @DbName		SYSNAME;
	DECLARE @RMDesc		VARCHAR (128);
	DECLARE @LastBack	VARCHAR (128);

	-- Take a copy of the master.sys.databases output as
	-- using a cursor directly on it was producing unpredictable
	-- results
	--
	SELECT * INTO msdb.dbo.SQLskillsSysDb
	FROM master.sys.databases
		WHERE [state_desc] ='ONLINE';

	-- Declare and open the cursor - this is easier
	-- than trying to use sp_MSforeachdb
	--
	DECLARE databases CURSOR FOR
		SELECT [name], [recovery_model_desc] FROM msdb.dbo.SQLskillsSysDb;

	OPEN databases;

	-- Loop through all online databases
	--
	FETCH NEXT FROM databases INTO @DbName, @RMDesc;

	WHILE @@FETCH_STATUS = 0
	BEGIN
		INSERT INTO msdb.dbo.SQLskillsLogInfo
		EXEC ('DBCC LOGINFO (''' + @DbName + ''') WITH NO_INFOMSGS');

		INSERT INTO msdb.dbo.SQLskillsDBCC
		EXEC ('DBCC DBINFO (''' + @DbName + ''') WITH TABLERESULTS, NO_INFOMSGS');

		select @FCount = COUNT (*)
		FROM sys.master_files
		WHERE [database_id] = DB_ID (@DbName)
		AND type_desc='LOG';

		SELECT @VLFs = COUNT (*)
		FROM msdb.dbo.SQLskillsLogInfo;

		SELECT @LRCount = [VALUE]
		FROM msdb.dbo.SQLskillsDBCC
		WHERE [Field] = 'dbi_RebuildLogs';

		SELECT @LastBack = [VALUE]
		FROM msdb.dbo.SQLskillsDBCC
		WHERE [Field] = 'dbi_LastLogBackupTime';

		UPDATE msdb.dbo.SQLskillsAllInfo
		SET [FileCount] = @FCount,
			[VLFCount] = @VLFs,
			[Rebuilds] = @LRCount,
			[RecModel] = @RMDesc,
			[LLBackup] = 
				(CASE WHEN (@LastBack LIKE '1900%')
					THEN 'NEVER' ELSE @LastBack END)
		WHERE [DatabaseName] = @DbName;

		TRUNCATE TABLE msdb.dbo.SQLskillsLogInfo;
		TRUNCATE TABLE msdb.dbo.SQLskillsDBCC;

		FETCH NEXT FROM databases INTO @DbName, @RMDesc;
	END

	-- Close and deallocate the cursor
	--
	CLOSE databases;
	DEALLOCATE databases;

	-- Output the results
	--
	SELECT 
		Upper(convert(varchar,SERVERPROPERTY('ServerName'))) as [server_name],
		SERVERPROPERTY('ProductVersion') as [product_version],
		SERVERPROPERTY('ProductLevel') as [product_level],
	[DatabaseName] AS [Database Name],
		[RecModel] AS [Recovery Model],
		ROUND ([LogSize], 2) AS [Log Size (MB)],
		ROUND ([LogUsed], 2) AS [Log Used (%)],
		[FileCount] AS [Log File Count],
		[VLFCount] AS [VLF Count],
		[Rebuilds] AS [Log Rebuilds],
		[LLBackup] AS [Last Log Backup]
	FROM msdb.dbo.SQLskillsAllInfo;

	-- Drop the temp tables
	--
	DROP TABLE msdb.dbo.SQLskillsLogSpace;
	DROP TABLE msdb.dbo.SQLskillsLogInfo;
	DROP TABLE msdb.dbo.SQLSkillsDBCC;
	DROP TABLE msdb.dbo.SQLskillsAllInfo;
	DROP TABLE msdb.dbo.SQLskillsSysDb;
END;
GO 
-- Mark the SP as a system object
--
EXEC sys.sp_MS_marksystemobject sp_SQLskillsAllLogInfo;
GO 

-- Run the script
--
EXEC sp_SQLskillsAllLogInfo;
GO

-- Clean up after ourselves
--
IF EXISTS (SELECT * FROM sys.objects WHERE NAME = 'sp_SQLskillsAllLogInfo')
	DROP PROCEDURE sp_SQLskillsAllLogInfo;
GO